// filename: goallist.h
//  Class definition for Goal List.

class GoalList  {
	friend class propnew;
	// friend class Proposition;
	friend class DepList;

	private:
		Goal	*firstptr, *lastptr;
	
	public:
		GoalList() {firstptr = lastptr = 0;}
		~GoalList();
		int isEmpty() {return firstptr == 0;}
		Goal *Searchg(int);
		void AddGoal(char*, JOINTYPE, GDCOLOR, GDLocation*);
		int maxgoal();
		void DelGoal(int);
		void gprint();
		void delgadj(int, OBJTYPE);
		void leaf(Dependency*);
		void geval(Dependency*);
		int visit();
		GDCOLOR combine(Goal *, Dependency*);
		GDCOLOR min(Goal *, Dependency*);
		GDCOLOR max(Goal *, Dependency*);
		GDCOLOR comb(Goal *, Dependency*);

};


//Destructor:
GoalList::~GoalList(void)
{

	while (firstptr != lastptr) {
		Goal	*tmp = firstptr;
		firstptr = firstptr->next;
	        delete tmp;
	}
}

Goal *GoalList::Searchg(int key)
{
	Goal	*tmp = firstptr;

	while (tmp != lastptr) {
		if (tmp->PKey == key) return tmp;
		else tmp = tmp->next;
	}
	return lastptr;
}


void GoalList::AddGoal(char *name, JOINTYPE type, GDCOLOR color,
			GDLocation *glocation)
{
	int gkey = maxgoal();
	Goal	*AddPtr = new Goal(gkey+1, name, type, color, glocation);
	if (firstptr == 0) {
	    firstptr = AddPtr;
	    firstptr->next = 0;
	}
	else {
		if (lastptr == 0) {
			firstptr->next = AddPtr;
			lastptr = AddPtr;
		}
		else {
			lastptr->next = AddPtr;
			lastptr = AddPtr;
		}
	}
}

int GoalList::maxgoal(void)
{
	int	maxkey;

	if (firstptr == 0)  {return maxkey =0;}
	else {
		if (firstptr->next ==0) { return firstptr->PKey;}
		else {
			Goal *tmp = firstptr;
			maxkey = tmp->PKey;
			while (tmp != lastptr->next) {
				if (maxkey <= tmp->PKey) {
					maxkey = tmp->PKey;
					tmp = tmp->next;
				}
			}
		}
	return maxkey;
	}
}

void GoalList::DelGoal(int key)
{
	Goal *sptr = Searchg(key);

	if (sptr != 0) {
	   if (sptr->AdjHdptr != 0) {
	       AObject *adjptr = sptr->AdjHdptr;
	       sptr->Dellink(adjptr);
	   }
	   Goal *tmp = firstptr;
	   if (tmp->PKey != sptr->PKey) {
	       while (tmp->next != sptr) {
		      tmp = tmp->next;
	       }
		tmp->next = sptr->next;
	  	delete sptr;

	   }
	   else { 
		firstptr = sptr->next;
		delete sptr; 
	  	}
	}
	
}


void GoalList::delgadj(int key, OBJTYPE objtype)
{
	Goal *dptr = firstptr;
	
	while (dptr != 0 ) {
		dptr->delobj(dptr, key, objtype);
		dptr = dptr->next;
	}
}


void GoalList::leaf(Dependency *dfirst)
{
	Goal *leafptr = firstptr;
	if (!isEmpty()) {
	    while (leafptr != 0) {
		if (leafptr->AdjHdptr == 0 ) {
		    int leafkey = leafptr->getkey(leafptr);
		    GDCOLOR llabel = leafptr->label;
		    DepList *dlptr;
		    dlptr->changeadj(leafkey, llabel, dfirst);// change labels 	
					//of all deps with leafkey adjacent.
		    leafptr->visited = VTRUE;
		}
		leafptr = leafptr->next;
	    }
	}
}


void GoalList::geval(Dependency *dfirst)
{
	Goal *evalptr = firstptr;
	int adjkey = 0, affectag = 0;

	while (evalptr != 0) {
	  if (evalptr->visited == VFALSE) {
	    AObject *adjptr = evalptr->AdjHdptr;
	    while (adjptr != 0 ) {
		adjkey = evalptr->getadjkey(adjptr);
		DepList *affectptr;
		AFFECTIVE affective;
		if ((affective = affectptr->checkaffect(adjkey, dfirst))
			== INAFFECT) {
		  affectag = 1;
		  adjptr = 0;
		}
		else {
		  adjptr = evalptr->gotonext(adjptr);
		}
	    }
	    if (affectag == 0) {
		GDCOLOR newl = combine(evalptr, dfirst);
		DepList *dlptr;
		dlptr->changeadj(evalptr->PKey, newl, dfirst);
	    }
	  }
	  evalptr = evalptr->next;
	}	
}

int GoalList::visit(void)
{
	Goal *visitptr = firstptr;
	while (visitptr != 0) {
		if (visitptr->visited == VFALSE) {return 1;}
		else {visitptr = visitptr->next;}
	}
	return 0;
}

GDCOLOR GoalList::combine(Goal *combptr, Dependency *dfirst)
{
	Goal *ptr = combptr;
	GDCOLOR nlabel;
	switch (ptr->JType) {
		case AND:
			nlabel = min(combptr, dfirst);
			break;
		case OR:
			nlabel = max(combptr, dfirst);
			break;
		case NON:
			nlabel = comb(combptr, dfirst);
			break;
		default:
			cout << "incorrect JType";
			break;
	}
	ptr->visited = VTRUE;
	ptr->label = nlabel;
	// DepList *dlptr;
	// dlptr->changeadj(ptr->PKey, nlabel, dfirst);
	return (nlabel);
}

GDCOLOR GoalList::min(Goal *minptr, Dependency *dfirst)
{
	AObject *minadjptr = minptr->AdjHdptr;
	int adj0 = 3;

	while (minadjptr != 0) {
	  int adjkey = minptr->getadjkey(minadjptr);
	  DepList *deptr;
	  int adj1 = deptr->getlabel(adjkey, dfirst);
	  switch (adj1) {
		case 0:
			return (DENIED);
		case 1: case 2:
			if ((adj0 == 1 || adj0 == 2) && (adj0 != adj1)) {
			   cout << "determine label(0 denied, 1 undetermine, 2 conflict, 3 satisfied)";
			   cin >> adj0;
			}
			else {
			  if (adj0 >= adj1) {adj0 = adj1;}
			}
			break;
		case 3:
			break;
	  }
	  minadjptr = minptr->gotonext(minadjptr);
	}
	switch (adj0) {
		case 0:
			return(DENIED);
		case 1:
			return(UNDETERMINE);
		case 2:
			return(CONFLICT);
		case 3:
			return(SATISFIED);
	}
}



GDCOLOR GoalList::max(Goal *maxptr, Dependency *dfirst)
{
	AObject *maxadjptr = maxptr->AdjHdptr;
	int adj0 = 0;

	while (maxadjptr != 0) {
	  int adjkey = maxptr->getadjkey(maxadjptr);
	  DepList *deptr;
	  int adj1 = deptr->getlabel(adjkey, dfirst);
	  switch (adj1) {
		case 0:
			break;
		case 1: case 2:
			if ((adj0 == 1 || adj0 == 2) && (adj0 != adj1)) {
			   cout << "determine label(0 denied, 1 undetermine, 2 conflict, 3 satisfied)";
			   cin >> adj0;
			}
			else {
			  if (adj0 <= adj1) {adj0 = adj1;}
			}
			break;
		case 3:
			return (SATISFIED);
	  }
	  maxadjptr = maxptr->gotonext(maxadjptr);
	}
	switch (adj0) {
		case 0:
			return(DENIED);
		case 1:
			return(UNDETERMINE);
		case 2:
			return(CONFLICT);
		case 3:
			return(SATISFIED);
	}
}
	

					  
GDCOLOR GoalList::comb(Goal *combptr, Dependency *dfirst)
{
	AObject *combadjptr = combptr->AdjHdptr;
	int adj0 = 0;

	while (combadjptr != 0) {
	  int adjkey = combptr->getadjkey(combadjptr);
	  DepList *deptr;
	  int adj1 = deptr->getlabel(adjkey, dfirst);
	  switch (adj1) {
		case 0: case 3:
			if ((adj0 == 0 || adj0 == 3) && (adj0 != adj1)) {
			  cout << "determine label(0 denied, 1 undetermine, 2 conflict, 3 satisfied)";
			cin >> adj0;
			} 
			break;
		case 1:
			if (adj0 > adj1) { adj0 = adj1; }  
			break;
		case 2:
			return (CONFLICT);
		  }
	  combadjptr = combptr->gotonext(combadjptr);
	}
	switch (adj0) {
		case 0:
			return(DENIED);
		case 1:
			return(UNDETERMINE);
		case 2:
			return(CONFLICT);
		case 3:
			return(SATISFIED);
	}
}



void GoalList::gprint(void)
{
	Goal *tmp = firstptr;

	if (!isEmpty()) {
	    cout << "==GOAL==" << "=====DEPENDENCY=====" << endl;

	    while(tmp != 0) {
		cout << tmp->PKey << tmp->GName << "(";
		switch (tmp->label) {
		  case DENIED :
		       cout << "DENIED";
		       break;
		  case UNDETERMINE:
		       cout << "UNDETERMINE";
		       break;
		  case CONFLICT:
		       cout << "CONFLICT";
		       break;
		  case SATISFIED:
		       cout << "SATISFIED";
		       break;
		}

		cout << ")";
		AObject *adjptr = tmp->AdjHdptr;
		tmp->adjprint(adjptr);
		tmp = tmp->next;
		cout << endl;
		}
	}
	else cout << "The goal list is empty." << endl;
}	      


